#ifndef _MK3_
#define _MK3_
#include <velib/mk2/mk2.h>

/* 'B' command function definitions  */
#define MK3_BOOTLOAD_START_REPROGRAM			0x01
#define MK3_BOOTLOAD_GET_IDS					0x02
#define MK3_BOOTLOAD_GET_VERSIONS				0x03

typedef struct
{
	un16 bootFunctionVersion;	/**< Version of boot functions callable from the application */
	un16 bootReprogramVersion;	/**< Version of bootblock features (When in bootloader) */
	un16 bootBlockVersion;		/**< 15..4 = boot block type, 3..0  = revision
								   bootblock type is subdivided in families and single units
								   for instance:
								   0b000000xxxx:	(Multi,Panel,SWL etc which can all use the same bootblock source but have small adaptations
								   for the error loops and TX hardware.fundamental differences (MK2<-> Multi,Panel,SWL)
								   0b0000010000:	MK3
								*/
} Mk3BootVersions;

typedef struct
{
	un16 hid;					/**< Hardware ID */
	un16 keyId;					/**< Key ID */
	un32 tid;					/**< Target ID */
	un8 retriesRemaining;		/**< retries after Communication timeout, used by the firmware update process to determine if we have a valid application,
								 * in that case we can start the update from there, otherwise we need to reboot and start out of the bootloader. */
	veBool timedOut;			/**<  communication timeout */
} Mk3IDs;

void mk3SelfTest (Mk2Callback* callback);
void mk3SetHardwareLines (Mk2Callback* callback, veBool panel, veBool standBy);
void mk3GetBootVersions(Mk3BootVersions* versions, Mk2Callback* callback);
void mk3GetIDsFromBootloader(Mk3IDs* ids, Mk2Callback* callback, un8 nRetry);
void mk3StartReprogram(void);

#endif	/* _MK3_ */
