#ifndef _VEBUSFR_H_
#define _VEBUSFR_H_

#define IGNORE_VLL_CODE_BIT (1 << 4) /* VeBusSysConfInfo byte 7 */

/* Message IDs for VE.Bus frames that need to be handled */
#define VEBUS_RSP               0x20
#define VEBUS_CMD               0x21 /* Special config command ID */
#define VEBUS_MISC_FRAME        0x38 /* Miscellaneous Frame */
#define VEBUS_RSP_PANEL         0x40
#define VEBUS_RSP_AC_IN_CONFIG  0x41

/* TODO: We should rename VEBUS_CMD and VEBUS_RSP.
 * At the time of implementation it was assumed that all VEBUS commands started with 0x21
 */

/* The broadcast address for presence frames */
#define VEBUS_DEVICE_ADDRESS_BROADCAST 0x00000000

typedef enum{
	IDENT_REQUEST,
	IDENT_DISABLE,
	FLASH_LEDS,
	RESET_DEVICE,
	WRITE_BUS_CONFIG,
	GET_BUS_CONFIG,
	FREEZE_BUS,
	__UNUSED,
	GET_PHASE_INFO,
	PANEL_IDENT_REQUEST = 0x80,
	BLOCK_BUS_ACTIVITY = 0x90
} VebusCommand;

/* BLOCK_BUS_ACTIVITY followed by: flags, t
 *    flag bit 0 : Config devices
 *    1 : Panels
 *    2 : Other
 * An MK2 reacts to flags 0, 1 and 2 with complete silence
 * A BMS does so on the flags Panels and Other
 * An 1140 should react only on the flag 'Other'
 *
 * Timeout = t * 0,5s
 **/

typedef enum{
	IDENT_MULTI             = 0x01,
	BUS_CONFIG_MULTI,
	BUS_CONFIG_QUATTRO,
	BUS_CONFIG_INVERTER,
	PHASE_INFO_AC3,
	PHASE_INFO_AC2,
	PHASE_INFO_AC1,
	PHASE_INFO_AC0_1,
	PHASE_INFO_AC0_2,
	PHASE_INFO_AC0_3,
	PHASE_INFO_AC0_4,
	PHASE_INFO_DC,
	IDENT_COMPACT           = 0x11,
	IDENT_PANEL             = 0x80
} VebusPresenceResponse;

typedef enum{
	EXT_SIGNAL_PERIOD_TIME   = 0x00,
	AUX_RELAY_CMD,
	AUX_RELAY_STATUS,
	SET_SOC,
	SET_SOC_DISABLE_SOC_CALC,
	EXTRA_I_BAT_INFO_ENABLE_SOC_CALC,
	TOTAL_I_BAT_INFO_ENABLE_SOC_CALC,
} VebusMiscFrCmd;

#ifdef CFG_WITH_VEBUS_CONFIG_FRAMES
void vebusFrSetConfig(VebusDevice *dev);
void vebusFrGetConfig(VebusDevice *dev, Mk2Callback *callback);
#endif
void vebusFrSendIdentRequest(Mk2Handler handler);
void vebusFrSetIdentBehaviour(un32 uniqueNumber, veBool sendIdent, Mk2Handler *handler);
void vebusFrSendBlockBusActivity(Mk2Callback *callback, un8 time);
void vebusFrSendSoc(Mk2Callback *callback, un16 soc);
void vebusFrSendIBatExtra(Mk2Callback *callback, sn32 iBat);
#endif	/* _VEBUSFR_H_ */
