#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <velib/platform/console.h>
#include <velib/utils/ve_logger.h>

#include <product_info.h>
#include <storage.h>
#include <task.h>

struct option consoleOptions[] =
{
	{"settings",		required_argument,	0,							'S'},
	{"type",			required_argument,	0,							't'},
#if NMEA2K
	{"unique_number",	required_argument,	0,							'n'},
#endif
	{0,					0,					0,							0}
};


static veBool mk2SetProductTypeByName(const char *name)
{
	if(strncmp(name,"mk2",3) == 0)
		mk2SetProductType(MK2);
	else if(strncmp(name,"mk3",3) == 0)
		mk2SetProductType(MK3);
	else {
		printf("error - Unsupported VE.Bus interface product\n");
		return veFalse;
	}

	return veTrue;
}

void consoleUsage(char* program)
{
	printf("usage %s\n", program);
	printf("\n");
	printf("PROGRAM OPTIONS:\n");
	printf("  -S, --settings\n");
	printf("    location of the settings file (defaults to settings.bin)\n");
	printf("\n");
	printf("  -t, --type\n");
	printf("    VE.bus interface product type. Supported types: \"mk2\",\"mk3\"\n");
	printf("    Default = \"mk2\"\n");
	printf("\n");
	printf("CAN-bus OPTIONS:\n");
	printf("  -n, --unique_number\n");
	printf("\n");
	pltOptionsUsage();
	printf("Victron Energy B.V.\n");
}

veBool consoleOption(int flag)
{
	switch(flag)
	{
#if NMEA2K
	case 'u':
		{
			un32 serial;
			serial = atoi(optarg);
			if (serial >= 999)
			{
				printf("serial number must be numeric and less then 999\n");
				return veTrue;
			}

			prodInfo.nameLow |= serial;
			return veTrue;
		}
#endif
	case 'S':
		storageSetFilename(optarg);
		return veTrue;
	case 't':
		return mk2SetProductTypeByName(optarg);
	}

	return veTrue;
}

veBool consoleArgs(int argc, char *argv[])
{
	VE_UNUSED(argv);

	if (argc != 0)
	{
		printf("error - no arguments are expected - missing a '-' or '--'?");
		return veFalse;
	}

	return veTrue;
}
