#include <velib/velib_inc_mk2.h>
#include <velib/mk2/mk2_remote_device.h>
#include <velib/mk2/values_float.h>
#include <velib/utils/ve_logger.h>

#include <values_nmea2k.h>
#include <dev_reg.h>

struct VebusSystemS vebusSystem;
struct Mk2S mk2;

void mk2FhRawFrameReceivedEvent(Mk2MsgRx* mk2msg)
{
#if CFG_WITH_MK2_TRACE
	mk2Dump("<", mk2msg);
#endif
}

void mk2FhFrameSendEvent(Mk2MsgTx* mk2msg)
{
	VE_UNUSED(mk2msg);
}

void mk2FhFrameReceivedEvent(Mk2MsgRx* mk2msg)
{
	VE_UNUSED(mk2msg);

#if CFG_WITH_CONTROL
	vebusControlHandler(MK2_FRAME, mk2msg);
#endif

#if CFG_WITH_MK2_REMOTE_DEVICE
	mk2RemoteDevForwardFrame(mk2msg);
#endif
}

/* Number of cells */
un8 mk2GetNoc(struct VebusDeviceS* dev)
{
	VE_UNUSED(dev);

	return 12;
}

/* Todo */
veBool mk2IsUS(struct VebusDeviceS* dev)
{
	VE_UNUSED(dev);

	return veFalse;
}

veBool mk2HasAcIn(VebusDevice* dev, un8 n)
{
	VebusSettingInfo* settingInfo = vebusGetSettingInfoOfDevice(dev);

	if (!settingInfo)
		return veFalse;

	switch (n)
	{
	case 0:
	case 1:
		return settingInfo[VE_I_MAINS_LIMIT_AC1].scale != 0;
	case 2:
		return settingInfo[VE_I_MAINS_LIMIT_AC2].scale != 0;
	default:
		return veFalse;
	}
}


un16 veGetCurrentLimitCb(void)
{
	un8 input = vebusActiveIn(&vebusSystem.acInConfig);

	if (input >= 2)
		return 0xFFFF;

#ifdef NMEA2K
	if (input == 0 && !(vebusN2k.remoteControlUsed & VE_REG_RC_AC1_IN_LIMIT))
		return 0xFFFF;

	if (input == 1 && !(vebusN2k.remoteControlUsed & VE_REG_RC_AC2_IN_LIMIT))
		return 0xFFFF;
#endif

	return devRegStored.currentLimit[input];
}
